/*
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 * 
 * Contact Information:
 * Dambach-Werke GmbH
 * Elektronische Leitsysteme
 * Fritz-Minhardt-Str. 1
 * 76456 Kuppenheim
 * Phone: +49-7222-402-0
 * Fax: +49-7222-402-200
 * mailto: info@els.dambach.de
 */

package de.bsvrz.sys.testsys.lib;

import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.FileReader;
import java.io.FileWriter;
import java.util.HashMap;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

/**
 * Kopiert ein Datei. Beim kopieren knnen Filter- und Ersetzungstexte angegeben werden.
 * @author Chriesten
 *
 */
public class KopiePat
{
  private String m_quellDatei = null; 
  private String m_zielDatei = null;
  private String m_matchPat = null;
  private String m_quellPat = null;
  private String m_ersatzFuerQuellPat = null; 


  public Boolean cpers() 
  {
    String aktion = "Fehler in cpers()";
    Boolean bret = true;
    
    try
    {
      cpersIntern(m_quellDatei, m_zielDatei, m_matchPat , m_quellPat , m_ersatzFuerQuellPat);
    }
    catch ( Exception e )
    {
      System.err.println(aktion + ": " + e.getMessage());
      e.printStackTrace();
      bret = false;
    }
    
    return(bret);
  }
  
  /**
   * Kopiert quellDatei nach zielDatei. Zeilen in quellDatei, die nicht matchPat enthalten werden gefiltert.
   * quellPat wird durch die Zeichenkette ersatzFuerQuellPat ersetzt.
   * 
   * @param quellDatei siehe obige Beschreibung
   * @param zielDatei siehe obige Beschreibung
   * @param matchPat siehe obige Beschreibung
   * @param quellPat siehe obige Beschreibung
   * @param ersatzFuerQuellPat siehe obige Beschreibung
   */
  private void cpersIntern(String quellDatei, String zielDatei, String matchPat, String quellPat, String ersatzFuerQuellPat) throws Exception 
  {
      BufferedReader in = new BufferedReader(new FileReader(quellDatei)); 
      BufferedWriter out = new BufferedWriter(new FileWriter(zielDatei)); 

      String line = "";
      while ( ((line=in.readLine()) != null) )
      {
        if ( (matchPat!=null) && (!matchPat.equals( "" )) && (!matchPat(line, matchPat))  )
        { // Zeile fortwerfen
          continue;
        }
        
        if ( (quellPat!=null) && (!quellPat.equals( "" )) )
        {
          line = ersetzePat(line, quellPat, ersatzFuerQuellPat);
        }
        out.write( line );
        out.newLine();
      }
      in.close();
      out.close();
  }
  
  /**
   * ersetzt alle Vorkommen von quellPat in quelle durch ersatzFuerQuellPat
   * @param quelle Zeichenkette in der ersetzt wird
   * @param quellPat zu ersetzendes Pattern
   * @param ersatzFuerQuellPat Ersatzzeichenkette
   */
  public String ersetzePat(String quelle, String quellPat, String ersatzFuerQuellPat)
  {
    String ziel;
    Pattern p = Pattern.compile(quellPat);
    Matcher m = p.matcher(quelle);
    StringBuffer sb = new StringBuffer();
    boolean found = m.find();
    while (found)
    {
      m.appendReplacement( sb, ersatzFuerQuellPat );
      found = m.find();
    }
    m.appendTail( sb );
    ziel = sb.toString();

    return(ziel);
  }

  /**
   * Liefert true, falls quellexp in quelle vorkommt ; ansonsten false. 
   * @param quelle zu durchsuchende Zeichenkette
   * @param quellPat SuchPattern
   * @return true, false siehe obige Beschreibung                                         
   */
  public boolean matchPat(String quelle, String quellPat)
  {
    Pattern p = Pattern.compile(quellPat);
    Matcher m = p.matcher(quelle);
    return(m.find());
  }

  /**
   * Auswertung der Modulparameter
   * 
   * @param args Argumente siehe main Methode
   * 
   */
  public Boolean parseArgumente( String[] args )
  {
    HashMap<String, String> schluesselWertPaare = ArgumentParser.parseArgStrings( args, "=" );

    m_quellDatei = ArgumentParser.getWert(schluesselWertPaare, "-quellDatei", true, false);
    m_zielDatei = ArgumentParser.getWert(schluesselWertPaare, "-zielDatei", true, false);
    m_matchPat = ArgumentParser.getWert(schluesselWertPaare, "-matchPat", false, false);
    m_quellPat = ArgumentParser.getWert(schluesselWertPaare, "-quellPat", false, false);

    m_ersatzFuerQuellPat = ArgumentParser.getWert(schluesselWertPaare, "-ersatzFuerQuellPat", false, false);
    if ( m_ersatzFuerQuellPat == null )
      m_ersatzFuerQuellPat = ArgumentParser.getWert(schluesselWertPaare, "-ers", false, false);

    if ( (m_quellDatei==null) || (m_zielDatei==null) )
    {
      System.out.println("Usage: parseArgumente -quellDatei=... -zielDatei=... [-matchPat=... -quellPat=... -ers=...]");
      return(false);
    }

    System.out.println(
        "m_quellDatei="+m_quellDatei+"\n"
        +"m_zielDatei="+m_zielDatei+"\n"
        +"m_matchPat="+m_matchPat+"\n"
        +"m_quellPat="+m_quellPat+"\n"
        +"m_ersatzFuerQuellPat="+m_ersatzFuerQuellPat
        );
    return(true);
  }

  
  /**
   * Testroutine ; testet ersetzePat()
   * 
   * @return true=OK, false=Fehler
   */
  public Boolean test_ersetzePat()
  {
    String quelle = "C1C2345C999";
    String quellexp = "[1-9]+";
    String ersatzFuerQuellPat = "AA";
    String ziel = ersetzePat( quelle, quellexp, ersatzFuerQuellPat );
    if ( !ziel.equals( "CAACAACAA" ) )
        return(false);

    quelle = "C1C2345C999";
    quellexp = "[abc]+";
    ersatzFuerQuellPat = "AA";
    ziel = ersetzePat( quelle, quellexp, ersatzFuerQuellPat );
    if ( !ziel.equals( quelle ) )
        return(false);
    
    return(true);
  }

  
  /**
   * Testroutine ; testet test_matchPat()
   * 
   * @return true=OK, false=Fehler
   */
  public Boolean test_matchPat()
  {
    Boolean bret; 
    bret = matchPat("Aabcr", "^[aA]");
    if ( !bret ) // must be True
      return(false);
    
    bret = matchPat("Aa333bcr", "[0-9]+");
    if ( !bret ) // must be True
      return(false);

    bret = matchPat("Aabcr", "[0-9]+");
    if ( bret ) // must be false
      return(false);
    
    return(true);
  }

  /**
   * Hauptprogramm
   * @param args folgende Programmargumente:  <BR>
   * -quellDatei: Datei die kopiert wird <BR>
   * -zielDatei: Datei auf die kopiert wird <BR>
   * -matchPat: Es werden nur Zeilen kopiert, die matchPat entsprechen <BR>
   * -quellPat: Dieses Pattern wird in den Kopiezeilen durch "ersatzFuerQuellPat" ersetzt <BR>
   * -ersatzFuerQuellPat: Ersatzzeichenkette. Siehe oben.
   */
  public static void main( String[] args )
{
    String[] myargs = new String[5];
    myargs[0] = "-quellDatei = c:/temp/schrottq.txt"; 
    myargs[1] = "-zielDatei =c:/temp/schrottz.txt";
    myargs[2] = "-matchPat=^[aA]"; 
    myargs[3] = "-quellPat=[0-9]+";
    myargs[4] = "-ersatzFuerQuellPat=eineZahl";
    
    KopiePat kp = new KopiePat();
    Boolean bret = kp.parseArgumente( args );
    if ( bret ) kp.cpers(); 

    if ( true ) return;

    if ( kp == null )
      kp = new KopiePat();
    System.out.println("ersPatOK?: "+kp.test_ersetzePat());
    System.out.println("matchPatOK?: "+kp.test_matchPat() );
    if ( true ) return;
}


}
