/**
 * 
 */
package de.bsvrz.sys.testsys.testsys;

import static org.junit.Assert.*;

import org.junit.After;
import org.junit.AfterClass;
import org.junit.Before;
import org.junit.BeforeClass;
import org.junit.Test;

import de.bsvrz.sys.funclib.application.StandardApplicationRunner;
import de.bsvrz.sys.testsys.testsys.IGibZugriff;
import de.bsvrz.sys.testsys.testsys.PluginVerwalter;
import de.bsvrz.sys.testsys.testsys.UeberpruefungSystemApp;

/**
 * @author Chriesten <br>
 * JUNIT Tests fr den Plugin Verwalter.
 * Geprft wird die
 * generische Erzeugung einzelner Plugin Klassen <br>
 * sowie die Methode ErzeugePlugins() der Klasse PluginVerwalter.
 *
 */
public class PluginVerwalterTest
{
  /**
   * Plginverwalter fuer Tests
   */
  PluginVerwalter m_pluginVerwalter = null;
  /**
   * @throws java.lang.Exception
   */
  @BeforeClass
  public static void setUpBeforeClass() throws Exception
  {
  }

  /**
   * @throws java.lang.Exception
   */
  @AfterClass
  public static void tearDownAfterClass() throws Exception
  {
  }

  /**
   * @throws java.lang.Exception
   */
  @Before
  public void setUp() throws Exception
  {
    m_pluginVerwalter = new PluginVerwalter();
  }

  /**
   * @throws java.lang.Exception
   */
  @After
  public void tearDown() throws Exception
  {
  }

  
  /**
   * Initialisierungsroutine, liefert Programmargumente (Main-Args), mit denen  
   * das Programm ausgefhrt werden soll
   * @return "Main Argumente" fr berprfung System
   */
  private String[] initArgs()
  {
    String[] myargs = new String[10];
    int argno = 0;
    myargs[0]="-datenverteiler=localhost:8083"; // lokaler Test DAV
    // myargs[argno++]="-datenverteiler=192.0.1.116:8085";  // globaler DAV
    String myDavServer = System.getenv( "DAVSERVER" ); // rechner:port
    if ( myDavServer != null )
      myargs[argno-1]="-datenverteiler="+myDavServer;  // globaler DAV

    myargs[argno++]="-debugLevelStdErrText=FINE";
    myargs[argno++]="-benutzer=Tester";
    // myargs[argno++]="-authentifizierung=C:/Datenverteiler/skripte-dosshell/passwd";
    myargs[argno++]="-authentifizierung=./passwd";
    // myargs[argno++] = "-modLog=C:/temp/TestTestSys/log<UHRZEIT>.log::6::BM|ERROR";
    myargs[argno++] = "-modLog=./log<UHRZEIT>.log::6::BM|ERROR";
    // plugins
    String plugUhrzeit = "de.bsvrz.sys.testsys.modulUhrzeit.ModulUhrzeit";
    String plugStartStopp = "de.bsvrz.sys.testsys.modulStartStoppUeb.ModulStartStoppUeb";
    String plugApplUeb = "de.bsvrz.sys.testsys.modulApplUeb.ModulApplUeb";

    // dummy bergeben
    myargs[argno++] = "-ueberpruefer=" + "XX";
    
    // Groesse aufs notwendige vermindern
    String[] myargs2 = new String[argno];
    for (int i = 0; i < argno; i++)
    {
      myargs2[i] = myargs[i];
    }
    myargs = null;
    myargs = myargs2;

    // Parameter ausgeben
    System.out.println("---------------------------------------");
    for (String arg : myargs)
    {
      System.out.println(arg);
    }
    System.out.println("---------------------------------------");
      
    
    return myargs2;
  }


  /**
   * Test methode for {@link PluginVerwalter#erzeugePlugins(String[], de.bsvrz.dav.daf.main.ClientDavInterface)}.
   * Test bergibt die 3 Standardplugins und prft deren erfolgreiche Erzeugung.
   */
  @Test
  public void testErzeugePlugins()
  {
    UeberpruefungSystemApp uesys = new UeberpruefungSystemApp();
    String args[] = initArgs();
    StandardApplicationRunner.run( uesys, args );
    m_pluginVerwalter = new PluginVerwalter();

    String[] plugins = new String[3];
    plugins[0] = "de.bsvrz.sys.testsys.modulUhrzeit.ModulUhrzeit";
    plugins[1] = "de.bsvrz.sys.testsys.modulStartStoppUeb.ModulStartStoppUeb";
    plugins[2] = "de.bsvrz.sys.testsys.modulApplUeb.ModulApplUeb";
    int ret = m_pluginVerwalter.erzeugePlugins( plugins, null );
    boolean ok = (ret==3);
    String mess = 3-ret + " Plugins konnten nicht erzeugt werden.";
    assertTrue( mess, ok );
  }
  
  
  /**
   * Test method for {@link de.bsvrz.sys.testsys.testsys.PluginVerwalter#genObject(java.lang.String, java.lang.String[], java.lang.String)}.
   * Pruefung, ob die 3 Standardplugins erzeugt werden knne und ob Plugins das 
   * verlanget Interface IMeldungFehler implementieren
   * DaV Verbindung wird nicht bentigt
   */
  @Test
  public void testGenObject()
  {
    String[] plugins = new String[3];
    plugins[0] = "de.bsvrz.sys.testsys.modulUhrzeit.ModulUhrzeit";
    plugins[1] = "de.bsvrz.sys.testsys.modulStartStoppUeb.ModulStartStoppUeb";
    plugins[2] = "de.bsvrz.sys.testsys.modulApplUeb.ModulApplUeb";
    for (int j = 0; j < plugins.length; j++)
    {
      Object pluginObj = m_pluginVerwalter.genObject( plugins[j], null, null );
      String mess = "Klasse \"" + plugins[j] + "\" kann nicht erzeugt werden";
      boolean ok = pluginObj!=null;
      assertTrue( mess, ok );
      if ( !ok )
      {
        System.err.println(mess);
        continue;
      }
      
      ok = pluginObj instanceof IGibZugriff;
      mess = "Klasse \"" + plugins[j] + "\" implementiert <<IGibZugriff>> nicht";
      assertTrue( mess, ok );
      if ( !ok )
        System.err.println(mess);
    }
  }

  public void myTestGenObject() throws Exception
  {
    testGenObject();
  }
  
  
  public void myTestErzeugePlugins() throws Exception
  {
    setUp();
    testErzeugePlugins();
  }

  public static void main(String args[]) throws Exception
  {
    PluginVerwalterTest te = new PluginVerwalterTest();
    String[] myargs = te.initArgs();
    // DaV initialisieren
    
    te.myTestErzeugePlugins();
  }
}
