/*
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 * 
 * Contact Information:
 * Dambach-Werke GmbH
 * Elektronische Leitsysteme
 * Fritz-Minhardt-Str. 1
 * 76456 Kuppenheim
 * Phone: +49-7222-402-0
 * Fax: +49-7222-402-200
 * mailto: info@els.dambach.de
 */

package de.bsvrz.sys.testsys.testsys.davlib;

import java.util.HashMap;
import java.util.Iterator;

import de.bsvrz.sys.testsys.lib.MyLogger;

/**
 * Hilfsklasse; definiert die Datentruktur zur Aufnahme der Rechner DAV Aspekt 
 * typ.rechner.rechnerInformation.Eigenschaft
 * 
 * @author ChriestenJ
 */
public class RechnerDaten
{
  // ---------------------------------------------------------------------------
  // Membervariablen
  // ---------------------------------------------------------------------------

  /**
   * PID des Rechners
   */
  public String m_rechnerPid = null;

  /**
   * Objektname im DAV
   */
  public String m_rechnerObjektName;
  
  /**
   * Rechner Name
   */
  public String m_name = null;

  /**
   * TCP/IP Adresse
   */
  public String m_tcpip = null;

  /**
   * Laueft Start/Stopp? Ergebnis der letzten Prfung.
   */
  public Boolean m_pruef_erg_startStoppOk = null;

  /**
   * aktueller Rechner?
   */
  public boolean m_ist_aktuell = false;
  
  // ---------------------------------------------------------------------------

  // ---------------------------------------------------------------------------
  // Konstruktoren
  // ---------------------------------------------------------------------------
  /**
   */
  public RechnerDaten()
  {
  }

  // ---------------------------------------------------------------------------

  /**
   * ueberschreibt die equals Methode
   * 
   * @param compObj
   *          Vergleichsobjekt
   * @return true (gleich) oder false (ungleich)
   */
  @Override
  public boolean equals( Object compObj )
  {
    if ( this == compObj )
      return true;

    if ( !(compObj instanceof RechnerDaten) )
      return false;
    // Alternative to the above line :
    if ( compObj == null || compObj.getClass() != this.getClass() )
      return false;

    // cast to native object is now safe
    RechnerDaten that = (RechnerDaten) compObj;

    return this.m_rechnerPid.equals( that.m_rechnerPid ) && this.m_name.equals( that.m_name )
        && this.m_tcpip.equals( that.m_tcpip );
  }

  // -----------------------------------------------------------------------------------------

  /**
   * Ausgabefunktion. Schreibt die aktuellen Objektdaten auf die Konsole.
   * 
   * @param titel
   *          Ueberschrift der Testausgabe
   */
  public void aus( String titel )
  {
    String aktuell = m_ist_aktuell ? "(*)" : "";
    MyLogger.logln("TTT", "rechnerPid=" + m_rechnerPid + aktuell + " ,name=" + m_name 
        + " ,TCPIP=" + m_tcpip + " ,startStoppOk=" + m_pruef_erg_startStoppOk 
        + ", ObjektName=" + m_rechnerObjektName);
  }

  /**
   * Ausgabe aller Rechner in der Liste alleDAVRechner.
   * @param alleDAVRechner
   * @param titel
   */
  public static void ausDavRechnerMap( HashMap<String, RechnerDaten> alleDAVRechner, String titel)
  {
    MyLogger.logln("TTT", "------------"+titel+"-------------------------------");
    for (Iterator<String> iterator = alleDAVRechner.keySet().iterator(); iterator.hasNext();)
    {
      String mykey = iterator.next();
      alleDAVRechner.get( mykey ).aus( "" );
    }
    MyLogger.logln("TTT", "-----------------------------------------------------");
  }

  /**
   * Testroutine fuer das lokale Modul. Wird fuer verschiedene Tests im Laufe des Entwicklungs- und Wartungsprozesses verwendet
   * 
   * @param args
   *          Testargumente
   */
  public static void main( String[] args )
  {
    RechnerDaten rechnerdat = new RechnerDaten();
    rechnerdat.m_rechnerPid = "recher";
    rechnerdat.m_name = "name";
    rechnerdat.m_tcpip = "tcpip";

    rechnerdat.aus( "tt" );
  }

}
